/*
 * audio.c - Milena TTS Web Service
 * Copyright (C) Bohdan R. Rau 2008 <ethanak@polip.com>
 * 
 * Milena is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * Milena is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with Milena.  If not, write to:
 * 	The Free Software Foundation, Inc.,
 * 	51 Franklin Street, Fifth Floor
 * 	Boston, MA  02110-1301, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/wait.h>

#include <milena.h>
#include <milena_mbrola.h>
#include "milena_ws.h"

struct milena *milena;
struct milena_mbrola_cfg *mbrola;
static int ex_flags;

void say(char *str,int (*callback)(char *,int))
{
	int n,ptype,pid;
	char ibuf[8192],obuf[8192];
	int pipe_out[2];
	int pipe_in[2];
	FILE *f;
	
	if (pipe(pipe_out)) {
		perror("pipe");
		exit(1);
	}
	if (pipe(pipe_in)) {
		perror("pipe");
		exit(1);
	}
	pid=fork();
	if (pid<0) {
		perror("fork");
		exit(1);
	}
	if (!pid) {
		char *args[32];
		int n;
		close(pipe_out[0]);
		close(pipe_in[1]);
		dup2(pipe_out[1],1);
		dup2(pipe_in[0],0);
		
		n=0;
		args[n++]="mbrola";
		args[n++]="-e";
		args[n++]="-f";
		args[n++]=pitch;
		args[n++]="-t";
		args[n++]=tempo;
		args[n++]=mbrola_voice;
		args[n++]="-";
		args[n++]="-";
		args[n++]=NULL;
		execv(mbrola_path,args);
		exit(1);
	}
	close(pipe_out[1]);
	close(pipe_in[0]);
	f=fdopen(pipe_in[1],"w");
	for (;;) {
		n=milena_GetPhrase(milena,&str,ibuf,8192,&ptype);
		if (n) break;
		milena_Prestresser(milena,ibuf,obuf,8192);
		milena_TranslatePhrase(milena,(unsigned char *)obuf,ibuf,8192,0);
		milena_Poststresser(ibuf,obuf,8192);
		milena_ModMbrolaGenPhrase(mbrola,obuf,f,ptype|ex_flags);
	}
	fprintf(f,"_ 10\n");
	fclose(f);
	for (;;) {
		char wave_buffer[8192];
		n=read(pipe_out[0],wave_buffer,8192);
		if (n<=0) break;
		if (callback(wave_buffer,n)) {
			kill(pid,9);
			break;
		}
	}
	close(pipe_out[0]);
	callback(NULL,0);
	waitpid(pid,NULL,0);
	
	
}

void start_milena(void)
{
	char obuf[256],ibuf[256],ixbuf[256];
	milena=milena_Init(
		milena_FilePath("pl_pho.dat",obuf),
		milena_FilePath("pl_dict.dat",ibuf),
		milena_FilePath("pl_stress.dat",ixbuf));
	if (!milena) exit(1);
	if (!milena_ReadPhraser(milena,
		milena_FilePath("pl_phraser.dat",ibuf))) exit(1);
	if (!milena_ReadPhraser(milena,
		milena_FilePath("pl_inet_phr.dat",ibuf))) exit(1);
	mbrola=milena_InitModMbrola(
		milena_FilePath("pl_mbrola.dat",ibuf));
	if (!mbrola) exit(1);
	if (!milena_ReadUserDic(milena,
		milena_FilePath("pl_udict.dat",ibuf))) exit(1);
	if (!milena_ReadUserDic(milena,
		milena_FilePath("pl_computer.dat",ibuf))) exit(1);
	if (!milena_ReadPhraser(milena,
		milena_FilePath("pl_hours.dat",ibuf))) exit(1);
	if (!milena_ReadInton(mbrola,
		milena_FilePath("pl_intona.dat",ibuf))) exit(1);
	ex_flags=0;
	if (conf_profi) {
		milena_ModMbrolaExtras(mbrola,
		milena_FilePath("pl_pro_mbrola.dat",ibuf));
		ex_flags =32;
		if (milena_ReadVerbs(milena,
				milena_FilePath("pl_verbs.dat",ibuf))) ex_flags|=64;
	}
}
